from django.core.management.base import BaseCommand
from wagtail.core.blocks import StructValue

from district.blocks import ProgramItemBlock
from district.models import DistrictProgramPage
from region.models import RegionProgramPage

from ...utils import get_issue_list, get_issue_overall_percentage


class Command(BaseCommand):
    def handle(self, *args, **options):
        """
        Pro updated_models (DistrictProgramPage, RegionProgramPage) projedu obsah,
        zda má v sobě `redmine_program_block` - ten má IntegerBlock `redmine_issue`.
        Podle něj pak stahujeme data z Redmine.
        """
        updated_models = DistrictProgramPage, RegionProgramPage
        self.stdout.write("Updating Redmine issues...")

        for model in updated_models:
            for page in model.objects.all():
                for program_block in page.content:
                    if program_block.block_type == "redmine_program_block":
                        fill_data_from_redmine(program_block.value)

                page.save()

        self.stdout.write("\nUpdating Redmine issues finished")


def fill_data_from_redmine(program_block_value: dict):
    """
    Naplní hondnotu completion_percentage a program_items pro RedmineProgramBlock
    """
    redmine_issue_id = program_block_value["redmine_issue"]
    fill_overall_percentage_from_redmine(program_block_value, redmine_issue_id)
    fill_program_items_from_redmine(program_block_value, redmine_issue_id)


def fill_overall_percentage_from_redmine(program_block_value: dict, issue_id: int):
    """
    Naplní hondnotu completion_percentage pro RedmineProgramBlock hodnoutou z Redmine.
    """
    program_block_value["completion_percentage"] = get_issue_overall_percentage(
        issue_id
    )


def fill_program_items_from_redmine(program_block_value: dict, issue_id: int):
    """
    Naplní hondnotu program_items pro RedmineProgramBlock. Nejdříve hodnotu (list)
    promaže a pak z getných issues naplní pomocí StructValue
    """
    issue_list = get_issue_list(issue_id)
    program_block_value["program_items"].clear()

    for issue in issue_list:
        sv = StructValue(ProgramItemBlock)
        sv.update(
            {
                "completion_percentage": issue.get("done_ratio", None),
                "title": issue.get("subject", ""),
            }
        )
        program_block_value["program_items"].append(sv)
